## Do not use the DES symmetric block cipher.

### Summary
The DES cipher was found, which is widely considered to be broken.

### Details
The [Data Encryption Standard](https://en.wikipedia.org/wiki/Data_Encryption_Standard), or DES, 
is a symmetric block cipher created in the 1970s. It has since been broken and should not be used
anywhere.

In general, the [Advanced Encryption Standard](https://en.wikipedia.org/wiki/Advanced_Encryption_Standard), 
or AES, algorithm, is preferred for all cases where symmetric encryption is needed.

#### Solution

##### .NET

Use the following method: `System.Security.Cryptography.Aes.Create()`

### Implementation

#### C# / .NET

DES is available through any of the following classes:

* `System.Security.Cryptography.DESCryptoServiceProvider`
* `System.Security.Cryptography.DES`

It can also be accessed through the following factory methods:

* `System.Security.Cryptography.SymmetricAlgorithm.Create("System.Security.Cryptography.DES")`
* `System.Security.Cryptography.SymmetricAlgorithm.Create("DES")`

#### C/C++ (Windows)

On Windows (C/C++), DES is available through the following:

* The algorithm identifier `CALG_DES` can be passed to various methods, including
  [CryptDeriveKey](https://msdn.microsoft.com/en-us/library/windows/desktop/aa379916(v=vs.85).aspx). 
* The algorithm identifier `BCRYPT_DES_ALGORITHM` can be passed to 
  [BCryptOpenAlgorithmProvider](https://msdn.microsoft.com/en-us/library/windows/desktop/aa375479(v=vs.85).aspx).

#### C/C++ (OpenSSL)

[OpenSSL](https://openssl.org) (and by extension, many libraries that wrap or pass through to
OpenSSL) makes the DES algorithm available through the following calls:

* `DES_*_encrypt` (e.g. DES_cbc_encrypt, DES_cfb_encrypt, etc.) - These are documented in 
  the [OpenSSL manpages](https://www.openssl.org/docs/manmaster/man3/DES_ofb_encrypt.html).
* Internally, the `DES_ENCRYPT` and `DES_DECRYPT` constants are passed to the `DES_*_encrypt`
  methods to specify whether the operation should encrypt plaintext or decrypt ciphertext.

### Severity Considerations

Any use of the DES algorithm should be very carefully reviewed by a security expert or cryptographer
to understand the risks involved.
